@@ -1,3 +1,4 @@
+
 #include "common_includes.h"
 
 RCSID ("$Id: filehandling_functions.c,v 1.28 2000/06/07 11:27:11 pborys Exp $")
@@ -35,19 +36,38 @@ Suffixes;
 {
   char *cbase = ((TagTable *) base)->nodename;
   char *ccompared = ((TagTable *) compared)->nodename;
-  while (*cbase)
+  return compare_tag_table_string (cbase, ccompared);
+}
+
+int 
+matchfile (char **buf, char *name)
+{
+#define Buf (*buf)
+  DIR *dir = opendir (Buf);
+  struct dirent *dp;
+  int namelen = strlen (name);
+  if (dir == NULL)
+    return 1;
+  while ((dp = readdir (dir)) != NULL)
     {
-      if (*cbase != *ccompared)
+      if (strncmp (dp->d_name, name, namelen) == 0)
 	{
-	  return (int) (*cbase) - (int) (*ccompared);
-	}
-      else
-	{
-	  cbase++;
-	  ccompared++;
+	  char *tmp = strdup (dp->d_name);
+	  int dl;
+	  strip_compression_suffix (tmp);
+	  dl = strlen (tmp);
+	  if (!isdigit (tmp[dl - 1]))	/* if it's not eg. info-2.gz, but info.gz, the primary page */
+	    {
+	      strcat (Buf, "/");
+	      strcat (Buf, tmp);
+	      closedir (dir);
+	      return 0;
+	    }
 	}
     }
-  return 0;
+  closedir (dir);
+  return 1;
+#undef Buf
 }
 
 FILE *
@@ -171,7 +191,7 @@ read_item (FILE * id, char **type, char ***buf, long *lines)
 					 * imprecise in combination with
 					 * some weird tag_tables).
 					 */
-  fgetc (id);			/* then skip the trailing `\n' */
+  while (fgetc (id) != '\n');	/* then skip the trailing `\n' */
 
   Type = xmalloc (1024);	/* read the header line */
   fgets (Type, 1024, id);
@@ -305,8 +325,18 @@ load_tag_table (char **message, long lines)
 	cut++;			/* increment the number of corrupt entries */
     }
   TagTableEntries = lines - 1 - is_indirect - cut;
-  FirstNodeOffset = tag_table[1].offset;
-  strcpy (FirstNodeName, tag_table[1].nodename);
+
+  /* FIXME: info should ALWAYS start at the 'Top' node, not at the first
+     mentioned node (vide ocaml.info) */
+
+  for (i = 1; i <= TagTableEntries; i++)
+    {
+      if (strcasecmp (tag_table[i].nodename, "Top") == 0)
+	{
+	  FirstNodeOffset = tag_table[i].offset;
+	  strcpy (FirstNodeName, tag_table[i].nodename);
+	}
+    }
   qsort (&tag_table[1], TagTableEntries, sizeof (TagTable), qsort_cmp);
 }
 int
@@ -571,6 +601,19 @@ opendirfile (int number)
   return NULL;
 }
 
+char *
+basename (char *filename)
+{
+  int len = strlen (filename);
+  char *a = filename + len;
+  while (a > filename)
+    {
+      a--;
+      if (*a == '/')
+	return a + 1;
+    }
+  return filename;		/* when it was a basename */
+}
 
 FILE *
 openinfo (char *filename, int number)	/*
@@ -586,18 +629,24 @@ openinfo (char *filename, int number)	/*
 					 * other info-subfiles.
 					 * The temporary file 1 is refrenced
 					 * by number=0, and file 2 by number=1
+					 * Openinfo by default first tries 
+					 * the path stored in char *filenameprefix
+					 * and then in the rest of userdefined
+					 * paths.
 					 */
 {
   FILE *id = NULL;
-  char buf[1024];		/* holds local copy of filename */
+  char *buf = xmalloc (1024);	/* holds local copy of filename */
   char *bufend;			/* points at the trailing 0 of initial name */
-  char *infosuffix;		/* it will keep the point of name, where `.info' starts */
   char command[1128];		/* holds command to evaluate for decompression of file */
   int i, j, twoloops;
   char *tmpfilename;
 
   if (strncmp (filename, "dir", 3) == 0)
-    return opendirfile (number);
+    {
+      xfree (buf);
+      return opendirfile (number);
+    }
 
   if (number == 0)		/* initialize tmp filename for file 1 */
     {
@@ -621,50 +670,120 @@ openinfo (char *filename, int number)	/*
       tmpfilename = tmpfilename2;	/* later we will refere only to tmp2 */
     }
 
-  for (twoloops = 0; twoloops < 2; twoloops++)	/* 
-						 * one loop with appended
-						 * `.info' suffix, and the
-						 * second without suffix
-						 */
+  for (i = -1; i < infopathcount; i++)	/* go through all paths */
     {
-      for (i = 0; i < infopathcount; i++)	/* go through all paths */
+      if (i == -1)
+	{
+	  if (!filenameprefix)
+	    continue;		/* no filenameprefix,
+				   we don't navigate around
+				   any specific infopage
+				   set, so simply scan all
+				   directories for a hit */
+	  else
+	    {
+	      strcpy (buf, filenameprefix);	/* build a filename */
+	      strcat (buf, "/");
+	      strcat (buf, basename (filename));
+	    }
+	}
+      else
 	{
 	  strcpy (buf, infopaths[i]);	/* build a filename */
-	  strcat (buf, "/");
-	  strcat (buf, filename);
-	  bufend = buf;		/* 
+	  if (matchfile (&buf, filename) == 1)	/* no match found in this directory */
+	    continue;
+	}
+      bufend = buf;		/* 
 				 * remember the bufend to make it
 				 * possible later to glue compression
 				 * suffixes.
 				 */
-	  bufend += strlen (buf);
-	  for (j = 0; j < SuffixesNumber; j++)	/* go through all suffixes */
-	    {
-	      strcat (buf, suffixes[j].suffix);
-	      if ((id = fopen (buf, "r")) != NULL)
-		{
-		  fclose (id);
-		  buildcommand (command, suffixes[j].command, buf, tmpfilename);
-		  system (command);
-		  id = fopen (tmpfilename, "r");
-		  if (id)
-		    return id;
-		}
-	      (*bufend) = 0;
-	    }
-	}
-      /* sometimes it happens that info files don't have .info suffix */
-      infosuffix = strstr (filename, ".info");
-      if (infosuffix)
+      bufend += strlen (buf);
+      for (j = 0; j < SuffixesNumber; j++)	/* go through all suffixes */
 	{
-	  *infosuffix = 0;
+	  strcat (buf, suffixes[j].suffix);
+	  if ((id = fopen (buf, "r")) != NULL)
+	    {
+	      fclose (id);
+	      clearfilenameprefix ();
+	      filenameprefix = strdup (buf);
+	      {			/* small scope for removal of filename */
+		int prefixi, prefixlen = strlen (filenameprefix);
+		for (prefixi = prefixlen; prefixi--; prefixi > 0)
+		  if (filenameprefix[prefixi] == '/')
+		    {
+		      filenameprefix[prefixi] = 0;
+		      break;
+		    }
+	      }
+	      buildcommand (command, suffixes[j].command, buf, tmpfilename);
+	      system (command);
+	      id = fopen (tmpfilename, "r");
+	      if (id)
+		{
+		  xfree (buf);
+		  return id;
+		}
+	    }
+	  (*bufend) = 0;
 	}
-      else
+      if ((i == -1) && (filenameprefix))	/* if we have a nonzero filename prefix,
+						   that is we view a set of infopages,
+						   we don't want to search for a page
+						   in all directories, but only in
+						   the prefix directory. Therefore
+						   break here. */
 	break;
     }
+  xfree (buf);
   return 0;
 }
 
+void
+addrawpath (char *filename)
+{
+  int len = strlen (filename);
+  int i, pos;
+  char tmp;
+  for (i = len; i >= 0; i--)
+    {
+      if (filename[i] == '/')
+	{
+	  tmp = filename[i];
+	  filename[i] = 0;
+	  pos = i;
+	  break;
+	}
+    }
+  if (i < 0)
+    pos = -1;
+
+  infopaths = xrealloc (infopaths, (infopathcount + 3) * (sizeof (char *)));
+  for (i = infopathcount; i > 0; i--)	/* move entries to the right */
+    infopaths[i] = infopaths[i - 1];
+
+  if (pos >= 0)
+    strcpy (infopaths[0], filename);	/* add new (raw) entry */
+  else
+    strcpy (infopaths[0], ".");
+  infopathcount++;
+
+  if (pos >= 0)			/* recreate original filename */
+    filename[pos] = tmp;
+}
+
+int
+isininfopath (char *name)
+{
+  int i;
+  for (i = 0; i < infopathcount; i++)
+    {
+      if (strcmp (name, infopaths[i]) == 0)
+	return 1;		/* path already exists */
+    }
+  return 0;			/* path not found in previous links */
+}
+
 void
 initpaths ()
 {
@@ -674,6 +793,9 @@ initpaths ()
 					 * and /usr/info were defined in $INFOPATH */
   int sharedir = 0;		/* the same for /usr/share/info */
   char *lang = getenv ("LANG");
+  char *infopath;
+  char *wsk;
+  char **tmpinfopaths;		/*for use for sorting */
 
   getcwd (curdir, 1024);
 
@@ -683,27 +805,32 @@ initpaths ()
 
   if (getenv ("INFOPATH") != NULL)	/* check paths in $INFOPATH variable */
     {
-      char *infopath = xmalloc (strlen (getenv ("INFOPATH")) + 100);
+      infopath = xmalloc (strlen (getenv ("INFOPATH")) + strlen (configuredinfopath) + 100);
       /* 
        * +5 (well, +100) of overrun buffer, since the following while loop
        *  can be coded simplier this way 
        */
-      char *wsk = infopath;
-      strcpy (infopath, getenv ("INFOPATH"));
-      infolen = strlen (infopath);
-      for (i = 0; i < infolen; i++)
-	if (infopath[i] == ':')
-	  infopath[i] = 0;
-      while (wsk < infopath + infolen)
+      wsk = infopath;
+      strcpy (infopath, getenv ("INFOPATH"));	/* copy the environmental path string */
+      strcat (infopath, ":");
+      strcat (infopath, configuredinfopath);	/* concatenate the paths from config file to the path string */
+    }
+  else
+    {
+      infopath = xmalloc (strlen (configuredinfopath) + 100);
+      wsk = infopath;
+      strcpy (infopath, configuredinfopath);
+    }
+  infolen = strlen (infopath);
+  for (i = 0; i < infolen; i++)
+    if (infopath[i] == ':')
+      infopath[i] = 0;
+  while (wsk < infopath + infolen)
+    {
+      infopaths = xrealloc (infopaths, (infopathcount + 2) * sizeof (char *));
+      wsklen = strlen (wsk);
+      if (!isininfopath (wsk))
 	{
-	  infopaths = xrealloc (infopaths, (infopathcount + 2) * sizeof (char *));
-	  wsklen = strlen (wsk);
-	  if (strncmp ("/usr/info", wsk, 9) == 0)
-	    globaldir = 1;
-	  if (strncmp ("/usr/local/info", wsk, 15) == 0)
-	    localdir = 1;
-	  if (strncmp ("/usr/share/info", wsk, 15) == 0)
-	    sharedir = 1;
 	  if (wsklen)
 	    {
 	      if (wsk[wsklen - 1] == '/')
@@ -712,12 +839,10 @@ initpaths ()
 	  if (lang)
 	    {
 	      infopaths[infopathcount] = xmalloc (wsklen + strlen (lang) + 2);
-	      strcpy (infopaths[infopathcount], wsk);
-	      strcat (infopaths[infopathcount], "/");
-	      strcat (infopaths[infopathcount], lang);
+	      sprintf (infopaths[infopathcount], "%s/%s", wsk, lang);	/* with $lang */
 	      infopathcount++;
 	      infopaths[infopathcount] = xmalloc (wsklen + 1);
-	      strcpy (infopaths[infopathcount], wsk);
+	      strcpy (infopaths[infopathcount], wsk);	/* without $lang */
 	      infopathcount++;
 	      infopaths = xrealloc (infopaths, sizeof (char *) * (infopathcount + 5));
 	    }
@@ -728,77 +853,28 @@ initpaths ()
 	      infopathcount++;
 	      infopaths = xrealloc (infopaths, sizeof (char *) * (infopathcount + 5));
 	    }
-	  wsk += wsklen + 1;	/*this will go to infopath[infolen+1]
-				   at the last token!!! */
 	}
-      xfree (infopath);
-      infopath = 0;
+      wsk += wsklen + 1;	/*this will go to infopath[infolen+1]
+				   at the last token!!! */
     }
+  xfree (infopath);
+  infopath = 0;
+
+  /* now we should sort our paths so that the national directories come first
+     before handling, they are interlaced:
+     .../info1/LANG|.../info1.../info2/LANG|.../info2|... */
+
   if (lang)
     {
-      infopaths = xrealloc (infopaths, sizeof (char *) * (infopathcount + 15));
-      if ((!globaldir) && (strcmp (curdir, "/usr/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (15 + strlen (lang));
-	  strcpy (infopaths[infopathcount], "/usr/info/");
-	  strcat (infopaths[infopathcount], lang);
-	  infopathcount++;
-	}
-      if ((!localdir) && (strcmp (curdir, "/usr/local/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (25 + strlen (lang));
-	  strcpy (infopaths[infopathcount], "/usr/local/info/");
-	  strcat (infopaths[infopathcount], lang);
-	  infopathcount++;
-	}
-      if ((!sharedir) && (strcmp (curdir, "/usr/share/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (25 + strlen (lang));
-	  strcpy (infopaths[infopathcount], "/usr/share/info/");
-	  strcat (infopaths[infopathcount], lang);
-	  infopathcount++;
-	}
-      if ((!globaldir) && (strcmp (curdir, "/usr/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (15);
-	  strcpy (infopaths[infopathcount], "/usr/info");
-	  infopathcount++;
-	}
-      if ((!localdir) && (strcmp (curdir, "/usr/local/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (25);
-	  strcpy (infopaths[infopathcount], "/usr/local/info");
-	  infopathcount++;
-	}
-      if ((!sharedir) && (strcmp (curdir, "/usr/share/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (25);
-	  strcpy (infopaths[infopathcount], "/usr/share/info");
-	  infopathcount++;
-	}
-    }
-  else
-    {
-      infopaths = xrealloc (infopaths, sizeof (char *) * (infopathcount + 15));
-      if ((!globaldir) && (strcmp (curdir, "/usr/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (15);
-	  strcpy (infopaths[infopathcount], "/usr/info");
-	  infopathcount++;
-	}
-      if ((!localdir) && (strcmp (curdir, "/usr/local/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (25);
-	  strcpy (infopaths[infopathcount], "/usr/local/info");
-	  infopathcount++;
-	}
-      if ((!sharedir) && (strcmp (curdir, "/usr/share/info")))
-	{
-	  infopaths[infopathcount] = xmalloc (25);
-	  strcpy (infopaths[infopathcount], "/usr/share/info");
-	  infopathcount++;
-	}
+      tmpinfopaths = xmalloc (infopathcount * sizeof (char *));
+      for (i = 1; i < infopathcount; i += 2)
+	tmpinfopaths[(int) (i / 2) + infopathcount / 2] = infopaths[i];
+      for (i = 0; i < infopathcount; i += 2)
+	tmpinfopaths[(int) (i / 2)] = infopaths[i];
+      xfree (infopaths);
+      infopaths = tmpinfopaths;
     }
+
   infopaths[infopathcount] = xmalloc (5);
   strcpy (infopaths[infopathcount], ".");	/* for ./ */
   infopathcount++;
@@ -842,7 +918,7 @@ create_tag_table (FILE * id)
     {
       if (fgetc (id) == INFO_TAG)	/* We've found a node entry! */
 	{
-	  fgetc (id);		/* skip '\n' */
+	  while (fgetc (id) != '\n');	/* skip '\n' */
 	  TagTableEntries++;	/* increase the nuber of tag table entries */
 	  oldpos = ftell (id);	/* remember this file position! */
 	  if (fgets (buf, 1024, id) == NULL)	/* 
