@@ -75,7 +75,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 				 */
     {
       pos = aftersearch;	/* set pos to the found position */
-/*      aftersearch=0; */
+/*      aftersearch=0;  * don't reset this--we want to know if we mus highlight something */
     }
 
   if (ncursor != -1)
@@ -105,12 +105,18 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	{
 	  if (statusline == FREE)
 	    showscreen (Message, Type, Lines, pos, cursor);
-	  nodelay (stdscr, FALSE);
+	  waitforgetch ();
 	  key = pinfo_getch ();
 	}
-      else
-	nodelay (stdscr, FALSE);
+      nodelay (stdscr, FALSE);
       statusline = FREE;
+      if (winchanged)		/* SIGWINCH */
+	{
+	  handlewinch ();
+	  winchanged = 0;
+	  addtopline (Type);
+	  key = pinfo_getch ();
+	}
 /***************************** keyboard handling ****************************/
       if (key != 0)
 	{
@@ -192,7 +198,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		      newpos -= (maxy - 1);
 		      if ((newpos > 0) && (newpos < Lines - (maxy - 2)))
 			pos = newpos;
-		      else if (newpos > 0)
+		      else if ((newpos > 0) && ((Lines - (maxy - 2)) > 0))
 			pos = Lines - (maxy - 2);
 		      else
 			pos = 1;
@@ -502,6 +508,13 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	      xfree (token);
 	      token = 0;
 
+	      if (!aftersearch)
+		{
+		  attrset (bottomline);
+		  mvaddstr (maxy - 1, 0, _ ("Search string not found..."));
+		  statusline = LOCKED;
+		}
+
 	      if (!aftersearch)
 		aftersearch = tmpaftersearch;
 
@@ -521,6 +534,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	  if ((key == keys.search_1) ||		/* search in current node */
 	      (key == keys.search_2))
 	    {
+	      int success = 0;
 	      move (maxy - 1, 0);
 	      attrset (bottomline);
 	      echo ();
@@ -560,6 +574,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		  strcat (tmp, Message[i + 1]);
 		  if (pinfo_re_exec (tmp))	/* execute the search command */
 		    {		/* if found, enter here */
+		      success = 1;
 		      if (pinfo_re_exec (Message[i + 1]))	/* 
 								   * if token was found
 								   * in the second line,
@@ -575,6 +590,7 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 			pos = i;
 		      xfree (tmp);	/* free tmp buffer */
 		      tmp = 0;
+		      aftersearch = 1;
 		      break;
 		    }
 		  else
@@ -584,10 +600,15 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 		      tmp = 0;
 		    }
 		}
+	      if (!success)
+		{
+		  attrset (bottomline);
+		  mvaddstr (maxy - 1, 0, _ ("Search string not found..."));
+		  statusline = LOCKED;
+		}
 	      xfree (token);	/* free user's search token */
 	      token = 0;
 	      rescan_cursor ();	/* rescan cursor position in the new place */
-	      aftersearch = 1;
 	    }
 	skip_search:
 /*==========================================================================*/
@@ -755,6 +776,10 @@ work (char ***message, char **type, long *lines, FILE * id, int tag_table_pos)
 	    {
 	      token = xmalloc (strlen (Type));
 	      getupnode (Type, token);
+	      if (strncmp (token, "(dir)", 5) == 0)
+		{
+		  ungetch (keys.dirpage_1);
+		}
 	      return_value = gettagtablepos (token);
 	      xfree (token);
 	      token = 0;
