@@ -113,12 +113,26 @@ set_initial_history (char *name)	/*
 					 * manual pages.
 					 */
 {
+  int len = strlen (name), i;
+  char *name1 = strdup (name);
   manualhistory = xmalloc (sizeof (manhistory));	/* one object of
 							   array */
-  strcpy (manualhistory[0].name, name);		/* filename->name */
+  while ((len > 1) && (isspace (name1[len - 1])))	/* filter trailing spaces */
+    {
+      name1[len - 1] = 0;
+      len--;
+    }
+  i = len;
+  for (i = len - 1; (i > 0) && (!isspace (name1[i])); i--);	/* find the beginning of the last token */
+
+  if (i > 0)
+    i++;			/* if we've found space, then we move to the first nonspace character */
+
+  strcpy (manualhistory[0].name, &name1[i]);	/* filename->name */
   strcpy (manualhistory[0].sect, "");	/* section unknown */
   manualhistory[0].selected = -1;	/* selected unknown */
   manualhistory[0].pos = 0;	/* pos=0 */
+  free (name1);
 }
 
 void
@@ -187,6 +201,7 @@ handlemanual (char *name)	/*
   char location[256];
   char line[1025];
   char *raw_tempfilename = 0;
+  char *apropos_tempfilename = 0;
   char *end, *prev;
   size_t macroline_size;
   int ignored_items = 0, i = 0;
@@ -214,10 +229,15 @@ handlemanual (char *name)	/*
   maxx = 80;
   maxy = 25;			/* otherwise hardcode 80x25... */
 #endif /* getmaxyx */
-
+#ifdef NIETS
 /****************************************************************************
  *                    Ignore macros part: BEGIN                             *
  * PS: Siewca: I still expect that you'll isolate it to a single procedure  *
+ * Description (by PB): This code opens a manpage file, and filters it from *
+ * dangerous macros. The output is put into a temporary file, which is then *
+ * used as the `name' filename argument of this (handlemanual) procedure.   *
+ * There is a stored variable raw_tempfilename to allow unlinking this temp *
+ * file after usage							    *
  ****************************************************************************/
   if (ignoredmacros)		/* if the pointer is non-null */
     if (*ignoredmacros && strlen (ignoredmacros))	/* if there are some macros */
@@ -352,30 +372,45 @@ handlemanual (char *name)	/*
 /****************************************************************************
  *                    Ignore macros part: END                               *
  ****************************************************************************/
-
-  snprintf (cmd, 255, "man %s %s %s > %s",
-	    ManOptions,
-	    name,
-	    StderrRedirection,
-	    tmpfilename1);
-  if (system (cmd) != 0)
+#endif
+  if (!plain_apropos)
+    snprintf (cmd, 255, "man %s %s %s > %s",
+	      ManOptions,
+	      name,
+	      StderrRedirection,
+	      tmpfilename1);
+  if ((plain_apropos) || (system (cmd) != 0))
     {
-      printf (_ ("Error: No manual page found either.\n"));
+      if (!plain_apropos)
+	{
+	  unlink (tmpfilename1);
+	  printf (_ ("Error: No manual page found\n"));
+	}
+      plain_apropos = 0;
       if (use_apropos)
 	{
-	  printf (_ ("Appropriate pages:\n"));
-	  snprintf (cmd, 255, "apropos %s|cat %s", name, StderrRedirection);
-	  system (cmd);
+	  printf (_ ("Calling apropos \n"));
+	  apropos_tempfilename = tempnam ("/tmp", NULL);
+	  snprintf (cmd, 255, "apropos %s > %s", name, apropos_tempfilename);
+	  if (system (cmd) != 0)
+	    {
+	      printf (_ ("Nothing apropiate\n"));
+	      unlink (apropos_tempfilename);
+	      return 1;
+	    }
+	  id = fopen (apropos_tempfilename, "r");
 	}
-      return 1;
+      else
+	return 1;
     }
-  id = fopen (tmpfilename1, "r");
+  else
+    id = fopen (tmpfilename1, "r");
   init_curses ();
 
 
+  set_initial_history (name);
   loadmanual (id);		/* load manual to memory */
   fclose (id);
-  set_initial_history (name);
   do
     {
       return_value = manualwork ();	/* manualwork handles all
@@ -416,6 +451,13 @@ handlemanual (char *name)	/*
 	    /* key_back was pressed */
 	    {
 	      manualhistorylength--;
+	      if (manualhistorylength == 0 && apropos_tempfilename)
+		{
+		  id = fopen (apropos_tempfilename, "r");
+		  loadmanual (id);
+		  fclose (id);
+		  continue;
+		}
 	      if (manualhistory[manualhistorylength].sect[0] == 0)
 		snprintf (cmd, 255, "man %s %s %s > %s",
 			  ManOptions,
@@ -470,6 +512,8 @@ handlemanual (char *name)	/*
 	}
     }
   while (return_value != -1);
+  if (apropos_tempfilename)
+    unlink (apropos_tempfilename);
   if (raw_tempfilename)
     unlink (raw_tempfilename);	/* we were using temporary */
   return 0;			/* raw-manpage for scaning */
@@ -685,7 +729,21 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
 			  break;
 			}
 		    }
-		  /* now we have needed string in i..breakpos. We need now to realloc the manuallinks table to make free space for new entry */
+		  /* now we have needed string in i..breakpos. We need now to realloc the
+		     manuallinks table to make free space for new entry */
+
+		  if (!((use_apropos) && (manualhistorylength == 0)))	/* a small check */
+		    {
+		      if ((!strcasecmp (&tmp[i], manualhistory[manualhistorylength].name))
+			  && ((!strcasecmp (p_t1, manualhistory[manualhistorylength].sect))
+			|| (manualhistory[manualhistorylength].sect[0] == 0)
+			      || (!strcmp (manualhistory[manualhistorylength].sect, " "))))
+
+			/* In English: if the name of the link is the name of the current page
+			   and the section of the link is the current section or if we don't
+			   know the current section, then... */
+			break;
+		    }
 		  manuallinks = xrealloc (manuallinks, sizeof (manuallink) * (ManualLinks + 3));
 		  manuallinks[ManualLinks].line = ManualLines;
 		  manuallinks[ManualLinks].col = i;
@@ -710,7 +768,7 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
 						 * the first word of this line */
 		    {
 		      if (b > 0)
-			if (!isspace (tmp[b] != ' '))
+			if (!isspace (tmp[b]))
 			  break;
 		    }
 		  if (b >= 0)
@@ -718,7 +776,7 @@ man_initializelinks (char *tmp, int carry)	/* initializes hyperlinks in manual *
 		  else
 		    manuallinks[ManualLinks].carry = carry;
 		  ManualLinks++;	/* increase the number of entries */
-		}
+		}		/*... if(in man links) */
 	      xfree ((void *) p_t1);
 	    }
 	}
@@ -743,6 +801,7 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
   char *tmp;			/* again the same */
   int key = 0;			/* key, which contains the value entered by user */
   int i, selectedchanged;	/* tmp values */
+  int statusline = FREE;
 #ifdef getmaxyx
   getmaxyx (stdscr, maxy, maxx);	/* if ncurses, get maxx and maxy */
   if ((!getenv ("MANWIDTH")) || (manwidthChanged))
@@ -777,13 +836,20 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
       key = pinfo_getch ();	/* action -- return ERR */
       if (key == ERR)		/* if there was nothing in buffer */
 	{
-	  showmanualscreen ();	/* then show screen */
-	  nodelay (stdscr, FALSE);
+	  if (statusline == FREE)
+	    showmanualscreen ();	/* then show screen */
+	  wrefresh (stdscr);
+	  waitforgetch ();
+	  key = pinfo_getch ();
+	}
+      nodelay (stdscr, FALSE);
+      statusline = FREE;
+      if (winchanged)
+	{
+	  handlewinch ();
+	  winchanged = 0;
 	  key = pinfo_getch ();
 	}
-      else
-	nodelay (stdscr, FALSE);
-
 /************************ keyboard handling **********************************/
       if (key != 0)
 	{
@@ -918,6 +984,7 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 	  if ((key == keys.search_1) ||		/* search in current node */
 	      (key == keys.search_2))
 	    {
+	      int success = 0;
 	      move (maxy - 1, 0);	/* procedure of getting regexp string */
 	      attrset (bottomline);
 	      echo ();
@@ -970,6 +1037,7 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 
 		  if (pinfo_re_exec (tmp))	/* execute search */
 		    {		/* if found, enter here... */
+		      success = 1;
 		      strcpy (tmp, manual[i + 1]);
 		      strip_manual (tmp);
 		      if (pinfo_re_exec (tmp))	/* 
@@ -986,6 +1054,13 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		}
 	      xfree (token);
 	      rescan_selected ();
+	      if (!success)
+		{
+		  attrset (bottomline);
+		  mvaddstr (maxy - 1, 0, _ ("Search string not found..."));
+		  statusline = LOCKED;
+		}
+
 	      manual_aftersearch = 1;
 	    }
 /*===========================================================================*/
@@ -1031,7 +1106,7 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		}
 	      if (!selectedchanged)	/* if new link not found */
 		{
-		  if (manualpos > 1)	/* move one position up */
+		  if (manualpos >= 1)	/* move one position up */
 		    manualpos--;
 		  for (i = 0; i < ManualLinks; i++)	/* 
 							 * and scan for selected
@@ -1055,6 +1130,32 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 		manualpos = 0;
 	      selected = ManualLinks - 1;
 	    }
+/*===========================================================================*/
+	  if ((key == keys.nextnode_1) ||
+	      (key == keys.nextnode_2))
+	    {
+	      for (i = manualpos + 1; i < ManualLines; i++)
+		{
+		  if (manual[i][1] == 8)
+		    {
+		      manualpos = i;
+		      break;
+		    }
+		}
+	    }
+/*===========================================================================*/
+	  if ((key == keys.prevnode_1) ||
+	      (key == keys.prevnode_2))
+	    {
+	      for (i = manualpos - 1; i > 0; i--)
+		{
+		  if (manual[i][1] == 8)
+		    {
+		      manualpos = i;
+		      break;
+		    }
+		}
+	    }
 /*===========================================================================*/
 	  if ((key == keys.pgdn_1) ||
 	      (key == keys.pgdn_2))
@@ -1089,7 +1190,7 @@ manualwork ()			/* viewer function. Handles keyboard actions--main event loop */
 	      if (manualpos > (maxy - 1))
 		manualpos -= (maxy - 1);
 	      else
-		manualpos = 1;
+		manualpos = 0;
 	      rescan_selected ();
 	    }
 /*===========================================================================*/
@@ -1346,6 +1447,7 @@ showmanualscreen ()		/* show the currently visible part of manpage */
 #ifdef HAVE_BKGDSET
   bkgdset (0);
 #endif
+  attrset (normal);
   add_highlights ();		/* add highlights */
   attrset (bottomline);		/* draw bottomline with user informations */
   mymvhline (0, 0, ' ', maxx);
@@ -1419,6 +1521,7 @@ mvaddstr_manual (int y, int x, char *str)	/* print a manual line */
 #else
   myclrtoeol ();
 #endif
+  attrset (normal);
 #ifndef ___DONT_USE_REGEXP_SEARCH___
   if ((h_regexp_num) || (manual_aftersearch))
     {
@@ -1476,7 +1579,7 @@ add_highlights ()		/* add hyperobject highlights */
 		  getyx (stdscr, y, x);
 		  if (y > 2)
 		    {
-#define TestCh manual[ltline][ltlinelen]
+#define TestCh tmpstr[ltlinelen]
 		      if (ltlinelen > 2)
 			ltlinelen -= 3;
 		      while ((isalpha (TestCh)) || (TestCh == '.'))
@@ -1555,7 +1658,7 @@ is_in_manlinks (char *in, char *find)	/* checks if a construction, which
 	  if (!strcasecmp (token, find))
 #else
 	  if (!strcmp (token, find))
-#endif	  
+#endif
 	    {
 	      xfree ((void *) copy);
 	      return 0;
