@@ -1,6 +1,10 @@
 #ifndef __PARSE_CONFIG_H
 #define __PARSE_CONFIG_H
 
+#ifndef ___DONT_USE_REGEXP_SEARCH___
+#include <regex.h>
+#endif
+
 #define BOLD 1
 #define NO_BOLD 0
 #define BLINK 1
@@ -63,4 +67,10 @@ int parse_line (char *line);
 char *str_toupper (char *s);
 char *skip_whitespace (char *s);
 char *remove_quotes (char *str);
+
+#ifndef ___DONT_USE_REGEXP_SEARCH___
+extern regex_t *h_regexp;	/* regexps to highlight */
+extern int h_regexp_num;	/* number of those regexps */
+#endif
+
 #endif
