@@ -40,14 +40,17 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
   char **message = 0;		/* this will hold node's text */
   char *type = 0;		/* this will hold the node's header */
   int tag_table_pos = 1;
+  char *tmp;
 #ifdef HAVE_GETOPT_LONG
   static struct option long_options[] =
   {
     {"help", 0, 0, 'h'},
     {"version", 0, 0, 'v'},
     {"manual", 0, 0, 'm'},
+    {"file", 0, 0, 'f'},
     {"raw-filename", 0, 0, 'r'},
     {"apropos", 0, 0, 'a'},
+    {"plain-apropos", 0, 0, 'p'},
     {"cut-man-headers", 0, 0, 'c'},
     {"squeeze-manlines", 0, 0, 's'},
     {"dont-handle-without-tag-table", 0, 0, 'd'},
@@ -55,6 +58,7 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
     {"node", 1, 0, 'n'},
     {"long-manual-links", 0, 0, 'l'},
     {"clear-at-exit", 0, 0, 'x'},
+    {"rcfile", 1, 0, 1},	/* no one-letter shortcut :( */
     {0, 0, 0, 0}};
 #endif
   signal_handler ();		/* take care of SIGSEGV, SIGTERM, SIGINT */
@@ -64,9 +68,6 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
   inithistory ();
   if (verbose)
     printf ("Przemek's Info Viewer v%s\n", version);
-  initpaths ();
-  parse_config ();		/* read config information */
-  checksu ();
   if (argc == 1)		/* if no arguments were given */
     {
       id = openinfo ("dir", 0);
@@ -96,7 +97,7 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 
   do
     {
-      command_line_option = getopt_long (argc, argv, "hvmracsdtnlx", long_options, NULL);
+      command_line_option = getopt_long (argc, argv, "hvmfrapcsdtnlx", long_options, NULL);
       switch (command_line_option)
 	{
 	case 'x':
@@ -114,6 +115,14 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 	  pinfo_start_node = malloc (strlen (optarg) + 1);
 	  strcpy (pinfo_start_node, optarg);
 	  break;
+	case 1:		/* rcfile */
+	  if (!optarg)
+	    {
+	      printf (_ ("--rcfile option used without argument\n"));
+	      exit (1);
+	    }
+	  rcfile = strdup (optarg);
+	  break;
 	case 't':
 	  ForceManualTagTable = 1;
 	  break;
@@ -125,7 +134,9 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 		     "-v, --version                         version\n" \
 		     "-m, --manual                          use man page\n" \
 		"-r, --raw-filename                    use raw filename\n" \
+		  "-f, --file                            synonym for -r\n" \
 		     "-a, --apropos                         call apropos if nothing found\n" \
+	       "-p, --plain-apropos                   call only apropos\n" \
 		     "-c, --cut-man-headers                 cut out repeated man headers\n" \
 		     "-l, --long-manual-links               use long link names in manuals\n" \
 		     "-s, --squeeze-manlines                cut empty lines from manual pages\n" \
@@ -133,12 +144,15 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 		     "                                      tables\n" \
 		     "-t, --force-manual-tag-table          force manual detection of tag table\n" \
 	    "-x, --clear-at-exit                   clear screen at exit\n" \
-		     "    --node=nodename, --node nodename  jump directly to the node nodename\n"),
+		     "    --node=nodename, --node nodename  jump directly to the node nodename\n" \
+	    "    --rcfile=file, --rcfile file      use alternate rcfile\n"),
 		  argv[0]);
 	  exit (0);
 	case 'v':
 	  exit (0);
 	case 'm':
+	  parse_config ();	/* read config information */
+          checksu ();
 	  if (verbose)
 	    printf (_ ("Looking for man page...\n"));
 	  strcpy (filename, "");
@@ -148,15 +162,28 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 	      strcat (filename, " ");
 	    }
 	  exit (handlemanual (filename));
+	case 'f':
 	case 'r':
 	  strncpy (filename, argv[argc - 1], 200);
 	  checkfilename (filename);	/* security check */
+	  addrawpath (filename);
+	  tmp = filename + strlen (filename) - 1;
 	  strip_compression_suffix (filename);	/* later, openinfo automaticaly adds them */
-	  id = openinfo (filename, 0);	/* try without '.info' suffix */
+	  while ((tmp > filename) && (*tmp != '/'))
+	    tmp--;		/* get basename */
+	  if (*tmp == '/')
+	    tmp++;
+	  id = openinfo (tmp, 0);	/* and try it without '.info' suffix */
 	  break;
 	case 'a':
 	  use_apropos = 1;
 	  break;
+	case 'p':
+	  use_apropos = 1;
+	  plain_apropos = 1;
+	  strncpy (filename, argv[argc - 1], 200);
+	  exit (handlemanual (filename));
+	  break;
 	case 'c':
 	  CutManHeaders = 1;
 	  break;
@@ -171,6 +198,9 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
   while (command_line_option != EOF);
 /*****************************************************************************/
 #endif
+  parse_config ();		/* read config information */
+  checksu ();
+  initpaths ();
 
   if (argc > 1)
     {
@@ -196,11 +226,8 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
       strcpy (curfile, filename);
     }
 
-  if (id == NULL)		/* try to open given filename with appended `.info' */
-    {
-      strcat (filename, ".info");
-      id = openinfo (filename, 0);
-    }
+  id = openinfo (filename, 0);
+
   if (id == NULL)		/* try to lookup the name in dir file */
     {
       id = dirpage_lookup (&type, &message, &lines, filename, &pinfo_start_node);
@@ -318,7 +345,8 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 		{
 		  char *tmp;
 		  fclose (id);
-		  tmp = addinfosuffix (work_return_value.file);
+		  tmp = strdup (work_return_value.file);	/*tmp = addinfosuffix (work_return_value.file); */
+		  clearfilenameprefix ();
 		  id = openinfo (tmp, 0);
 		  xfree (tmp);
 		  tmp = 0;
@@ -336,8 +364,8 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 			  npos = infohistory.pos[infohistory.length];
 			  ncursor = infohistory.cursor[infohistory.length];
 			}
-		      strip_file_from_info_suffix (curfile);
-		      tmp = addinfosuffix (curfile);
+		      strip_file_from_info_suffix (curfile);	/* open back the old file */
+		      tmp = strdup (curfile);	/*tmp = addinfosuffix (curfile); */
 		      id = openinfo (tmp, 0);
 		      xfree (tmp);
 		      tmp = 0;
@@ -359,20 +387,21 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 		      curfile = xmalloc (strlen (work_return_value.file) + 150);
 		      strcpy (curfile, work_return_value.file);
 		      freeindirect ();
-		      if (seek_indirect (id))
+		      if (seek_indirect (id))	/* find the indirect entry */
 			{
-			  read_item (id, &type, &message, &lines);
-			  load_indirect (message, lines);
+			  read_item (id, &type, &message, &lines);	/* read it */
+			  load_indirect (message, lines);	/* initialize indirect entries */
 			}
-		      freetagtable ();
-		      if (seek_tag_table (id) != 2)
+		      freetagtable ();	/* free old tag table */
+		      if (seek_tag_table (id) != 2)	/* search for the new tagtable */
 			{
-			  if (ForceManualTagTable == 0)
+			  if (ForceManualTagTable == 0)		/* if no manual initialization requested, load the tag table */
 			    {
 			      read_item (id, &type, &message, &lines);
 			      load_tag_table (message, lines);
 			    }
 			  else
+			    /* create tag table manually */
 			    {
 			      if (indirect)
 				create_indirect_tag_table ();
@@ -384,6 +413,7 @@ RCSID (PKG_VER "$Id: pinfo.c,v 1.36 2000/08/27 19:09:48 pborys Exp $")
 			    }
 			}
 		      else
+			/* no tagtable found */
 			{
 			  if (!DontHandleWithoutTagTable)
 			    {
