@@ -45,6 +45,9 @@ RCSID ("$Id: readlinewrapper.c,v 1.12 2000/01/27 11:38:16 misiek Exp $")
 	  if (cursor < strlen (buf))
 	    cursor++;
 	  break;
+	case KEY_END:
+	  cursor = strlen (buf);
+	  break;
 	case KEY_BCKSPC:	/* handle backspace: copy all */
 	case KEY_BACKSPACE:	/* chars starting from curpos */
 	  if (cursor > 0)	/* - 1 from buf[n+1] to buf   */
@@ -83,6 +86,8 @@ RCSID ("$Id: readlinewrapper.c,v 1.12 2000/01/27 11:38:16 misiek Exp $")
 								 * input buf 
 								 */
 	      }
+	  if (cursor > strlen (buf))
+	    cursor = strlen (buf);
 	  break;
 	case KEY_DOWN:		/* forwards-history call */
 	  if (rlhistorylen)
@@ -91,6 +96,8 @@ RCSID ("$Id: readlinewrapper.c,v 1.12 2000/01/27 11:38:16 misiek Exp $")
 		rlhistorypos++;
 		strcpy (buf, rlhistory[rlhistorypos - 1]);
 	      }
+	  if (cursor > strlen (buf))
+	    cursor = strlen (buf);
 	  break;
 	  /* eliminate nonprintable chars */
 	case '\n':
@@ -129,10 +136,13 @@ RCSID ("$Id: readlinewrapper.c,v 1.12 2000/01/27 11:38:16 misiek Exp $")
 		}
 	    }
 	}
-      mvaddstr (origy, origx + strlen (buf), "   ");
       move (origy, origx);
-      printw (buf);
+      for (i = origx; i < maxx; i++)
+	addch (' ');
+      move (origy, origx);
+      addstr (buf);
       move (origy, origx + cursor);
+
     }
   strcpy (rlhistory[rlhistorylen - 1], buf);
   if (strlen (buf))
