@@ -9,7 +9,7 @@ RCSID ("$Id: signal_handler.c,v 1.10 2000/01/27 11:38:16 misiek Exp $")
        handle_crash (int signum)
 {
   closeprogram ();
-  fprintf (stderr, "Catched signal %d, bye!\n", signum);
+  fprintf (stderr, "Caught signal %d, bye!\n", signum);
   if (signum == SIGSEGV)
     perror ("pinfo: crash with");
   exit (1);
@@ -18,18 +18,8 @@ RCSID ("$Id: signal_handler.c,v 1.10 2000/01/27 11:38:16 misiek Exp $")
 void
 handle_window_resize (int signum)
 {
-#ifdef	USE_NCURSES
-  struct winsize winsz;
-  myendwin ();
-  ioctl (0, TIOCGWINSZ, &winsz);
-  COLS = winsz.ws_col;
-  LINES = winsz.ws_row;
-  init_curses ();
-#ifdef getmaxyx
-  getmaxyx (stdscr, maxy, maxx);
-#endif /* getmaxyx */
+  winchanged = 1;
   ungetch (keys.refresh_1);
-#endif /* getmaxyx */
   signal (SIGWINCH, handle_window_resize);
 }
 
